/*
 * Thumbnailator - a thumbnail generation library
 *
 * Copyright (c) 2008-2020 Chris Kroells
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
 * THE SOFTWARE.
 */

package net.coobird.thumbnailator.tasks.io;

import java.awt.image.BufferedImage;
import java.io.IOException;

/**
 * An {@link ImageSink} which stores the resulting thumbnail to a
 * {@link BufferedImage}.
 * 
 * @author coobird
 *
 */
public class BufferedImageSink extends AbstractImageSink<BufferedImage> {
	/**
	 * The {@link BufferedImage} which holds the thumbnail.
	 */
	private BufferedImage img;
	
	/**
	 * Indicates whether the thumbnail has been written to this object.
	 */
	private boolean written = false;
	
	public void write(BufferedImage img) throws IOException {
		super.write(img);
		
		this.img = img;
		written = true;
	}
	
	/**
	 * Returns the thumbnail.
	 * 
	 * @return							The thumbnail.
	 * @throws IllegalStateException	If a thumbnail has not been stored to
	 * 									this {@link BufferedImageSink} yet.
	 */
	public BufferedImage getSink() {
		if (!written) {
			throw new IllegalStateException("BufferedImageSink has not been written to yet.");
		}
		return img;
	}

	@Override
	public void setOutputFormatName(String format) {
		// do nothing
	}
}
